@@ -53,9 +53,6 @@ import org.slf4j.Logger;
 import com.hazelcast.core.HazelcastInstance;
 import com.openexchange.authentication.AuthenticationService;
 import com.openexchange.config.ConfigurationService;
-import com.openexchange.config.Interests;
-import com.openexchange.config.Reloadable;
-import com.openexchange.config.Reloadables;
 import com.openexchange.config.lean.LeanConfigurationService;
 import com.openexchange.context.ContextService;
 import com.openexchange.dispatcher.DispatcherPrefixService;
@@ -65,7 +62,7 @@ import com.openexchange.login.listener.LoginListener;
 import com.openexchange.mail.api.AuthenticationFailedHandler;
 import com.openexchange.net.ssl.SSLSocketFactoryProvider;
 import com.openexchange.oidc.OIDCBackend;
-import com.openexchange.oidc.http.outbound.HttpConfig;
+import com.openexchange.oidc.http.outbound.OIDCHttpClientConfig;
 import com.openexchange.oidc.hz.PortableAuthenticationRequestFactory;
 import com.openexchange.oidc.hz.PortableLogoutRequestFactory;
 import com.openexchange.oidc.impl.OIDCAuthenticationFailedHandler;
@@ -76,6 +73,8 @@ import com.openexchange.oidc.impl.OIDCSessionParameterNamesProvider;
 import com.openexchange.oidc.impl.OIDCSessionSsoProvider;
 import com.openexchange.oidc.spi.OIDCCoreBackend;
 import com.openexchange.osgi.HousekeepingActivator;
+import com.openexchange.rest.client.httpclient.HttpClientService;
+import com.openexchange.rest.client.httpclient.SpecificHttpClientConfigProvider;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.serverconfig.ServerConfigService;
 import com.openexchange.session.SessionSsoProvider;
@@ -86,6 +85,7 @@ import com.openexchange.sessiond.SessiondService;
 import com.openexchange.sessionstorage.SessionStorageParameterNamesProvider;
 import com.openexchange.sessionstorage.SessionStorageService;
 import com.openexchange.user.UserService;
+import com.openexchange.version.VersionService;
 
 /**
  * Activates the OpenID feature.
@@ -133,24 +133,12 @@ public class OIDCActivator extends HousekeepingActivator{
         trackService(SessionStorageService.class);
         trackService(LockService.class);
         trackService(SSLSocketFactoryProvider.class);
+        trackService(VersionService.class);
+        trackService(HttpClientService.class);
         openTrackers();
 
         // Initialize configuration for outbound HTTP traffic
-        {
-            HttpConfig.initInstance(getService(ConfigurationService.class));
-            registerService(Reloadable.class, new Reloadable() {
-
-                @Override
-                public void reloadConfiguration(ConfigurationService configService) {
-                    HttpConfig.initInstance(configService);
-                }
-
-                @Override
-                public Interests getInterests() {
-                    return Reloadables.interestsForProperties("com.openexchange.oidc.http.outbound.*");
-                }
-            });
-        }
+        registerService(SpecificHttpClientConfigProvider.class, new OIDCHttpClientConfig());
 
         OIDCConfigImpl config = new OIDCConfigImpl(this);
 
@@ -187,7 +175,10 @@ public class OIDCActivator extends HousekeepingActivator{
             this.oidcBackendRegistry = null;
             oidcBackends.close();
         }
-        HttpConfig.resetInstance();
+        HttpClientService httpClientService = getService(HttpClientService.class);
+        if (httpClientService != null) {
+            httpClientService.destroyHttpClient(OIDCHttpClientConfig.getClientIdOidc());
+        }
         Services.setServices(null);
         super.stopBundle();
     }
